var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function _extendableBuiltin(cls) {
  function ExtendableBuiltin() {
    var instance = Reflect.construct(cls, Array.from(arguments));
    Object.setPrototypeOf(instance, Object.getPrototypeOf(this));
    return instance;
  }

  ExtendableBuiltin.prototype = Object.create(cls.prototype, {
    constructor: {
      value: cls,
      enumerable: false,
      writable: true,
      configurable: true
    }
  });

  if (Object.setPrototypeOf) {
    Object.setPrototypeOf(ExtendableBuiltin, cls);
  } else {
    ExtendableBuiltin.__proto__ = cls;
  }

  return ExtendableBuiltin;
}

// external dependencies
import stringify from 'json-stringify-safe';
import hashIt from 'hash-it';
import { parse } from 'pathington';
import { get as _get, has as _has, merge as _merge, remove, set as _set } from 'unchanged';

// classes
import CrioObject from './CrioObject';

// constants
import { ARRAY_FALLBACK_PROTOTYPE_METHODS, ARRAY_UNSCOPABLES } from './constants';

// is
import { isArray, isCrio, isEqual, isUndefined } from './is';

// utils
import { createIterator, find, getCrioedObject, getEntries, getRelativeValue, getValues, thaw as _thaw } from './utils';

var hasAppliedPrototype = false;

var CrioArray = function (_extendableBuiltin2) {
  _inherits(CrioArray, _extendableBuiltin2);

  function CrioArray(array) {
    var _ret2;

    _classCallCheck(this, CrioArray);

    var _this = _possibleConstructorReturn(this, _extendableBuiltin2.call(this, isArray(array) ? array.length : array || 0));

    if (!hasAppliedPrototype) {
      applyPrototype();

      hasAppliedPrototype = true;
    }

    if (isCrio(array)) {
      var _ret;

      return _ret = array.toArray(), _possibleConstructorReturn(_this, _ret);
    }

    return _ret2 = isArray(array) ? array.reduce(function (crioArray, item, index) {
      crioArray[index] = getCrioedObject(item);

      return crioArray;
    }, _this) : _this, _possibleConstructorReturn(_this, _ret2);
  }

  /**
   * @function clear
   * @memberof CrioArray
   *folder
   * @description
   * get a new empty array
   *
   * @returns {CrioArray} the empty array
   */
  CrioArray.prototype.clear = function clear() {
    return new CrioArray();
  };

  /**
   * @function compact
   * @memberof CrioArray
   *
   * @description
   * get a new array with values from the original array that are truthy
   *
   * @returns {CrioArray} the array with only truthy values
   */


  CrioArray.prototype.compact = function compact() {
    return this.filter(function (item) {
      return !!item;
    });
  };

  /**
   * @function copyWithin
   *
   * @description
   * move values around within the array
   *
   * @param {number} targetIndex target to copy
   * @param {number} [startIndex=0] index to start copying to
   * @param {number} [endIndex=this.length] index to stop copying to
   * @returns {CrioArray} array with target copied in appropriate spots
   */


  CrioArray.prototype.copyWithin = function copyWithin(targetIndex) {
    var startIndex = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
    var endIndex = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : this.length;

    var clone = [].concat(this);
    var length = this.length >>> 0;

    var to = getRelativeValue(targetIndex >> 0, length),
        from = getRelativeValue(startIndex >> 0, length);

    var final = getRelativeValue(endIndex >> 0, length);

    var count = Math.min(final - from, length - to),
        direction = 1;

    if (from < to && to < from + count) {
      direction = -1;
      from += count - 1;
      to += count - 1;
    }

    while (count > 0) {
      if (from in clone) {
        clone[to] = clone[from];
      } else {
        delete clone[to];
      }

      from += direction;
      to += direction;
      count--;
    }

    return new this.constructor(clone);
  };

  /**
   * @function delete
   * @memberof CrioArray
   *
   * @description
   * delete the value in the array at key, either shallow or deep
   *
   * @param {Array<number|string>|number} key the key to delete
   * @returns {CrioArray} the array with the key deleted
   */


  CrioArray.prototype.delete = function _delete(key) {
    return remove(key, this);
  };

  /**
   * @function difference
   * @memberof CrioArray
   *
   * @description
   * find the values in this that do not exist in any of the arrays passed
   *
   * @param {...Array<Array<*>>} arrays arrays to get the difference of
   * @returns {CrioArray} array of items matching diffference criteria
   */


  CrioArray.prototype.difference = function difference() {
    for (var _len = arguments.length, arrays = Array(_len), _key = 0; _key < _len; _key++) {
      arrays[_key] = arguments[_key];
    }

    if (!arrays.length) {
      return this;
    }

    var indexOfItem = void 0;

    return arrays.reduce(function (differenceArray, array) {
      if (isArray(array)) {
        array.forEach(function (item) {
          indexOfItem = differenceArray.indexOf(item);

          if (~indexOfItem) {
            differenceArray = differenceArray.splice(indexOfItem, 1);
          }
        });
      }

      return differenceArray;
    }, this);
  };

  /**
   * @function entries
   * @memberof CrioArray
   *
   * @description
   * get the pairs of [key, value] in the crio
   *
   * @returns {CrioArray} [key, value] pairs
   */


  CrioArray.prototype.entries = function entries() {
    return getEntries(this);
  };

  /**
   * @function equals
   * @memberof CrioArray
   *
   * @description
   * does the object passed equal the crio
   *
   * @param {*} object object to compare against the instance
   * @returns {boolean} is the object equivalent in value
   */


  CrioArray.prototype.equals = function equals(object) {
    return isEqual(this, object);
  };

  /**
   * @function fill
   *
   * @description
   * fill the array at certain indices with the value passed
   *
   * @param {*} value the value to fill the indices with
   * @param {number} [startIndex=0] the starting index to fill
   * @param {number} [endIndex=this.length] the ending index to fill
   * @returns {CrioArray} array with values filled appropriately
   */


  CrioArray.prototype.fill = function fill(value) {
    var startIndex = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
    var endIndex = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : this.length;

    var from = startIndex < 0 ? this.length + startIndex : startIndex;
    var to = endIndex < 0 ? this.length + endIndex : endIndex;
    var crioedValue = getCrioedObject(value);

    return this.map(function (item, index) {
      return index >= from && index < to ? crioedValue : item;
    });
  };

  /**
   * @function findLast
   * @memberof CrioArray
   *
   * @description
   * find an item in the crio if it exists, starting from the end and iteratng to the start
   *
   * @param {function} fn function to test for finding the item
   * @returns {*} found item or undefined
   */


  CrioArray.prototype.findLast = function findLast(fn) {
    return find(this, fn, false, true);
  };

  /**
   * @function findLastIndex
   * @memberof CrioArray
   *
   * @description
   * find the matching index based on truthy return from fn starting from end
   *
   * @param {function} fn function to use for test in iteration
   * @returns {number} index of match, or -1
   */


  CrioArray.prototype.findLastIndex = function findLastIndex(fn) {
    return find(this, fn, true, true);
  };

  /**
   * @function first
   * @memberof CrioArray
   *
   * @description
   * take the first n number of items in the array
   *
   * @param {number} [size=1] size of elements to take from beginning of array
   * @returns {CrioArray}
   */


  CrioArray.prototype.first = function first() {
    var size = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 1;

    return this.slice(0, size);
  };

  /**
   * @function forEach
   * @memberof CrioArray
   *
   * @description
   * iterate over the array executing fn
   *
   * @param {function} fn the function to execute
   * @returns {CrioArray} the original array
   */


  CrioArray.prototype.forEach = function forEach(fn) {
    Array.prototype.forEach.call(this, fn);

    return this;
  };

  /**
   * @function get
   * @memberof CrioArray
   *
   * @description
   * get the item at key passed, either shallow or deeply nested
   *
   * @param {Array<number|string>|number} key key to retrieve
   * @returns {*} item found at key
   */


  CrioArray.prototype.get = function get(key) {
    return _get(key, this);
  };

  /**
   * @function has
   * @memberof CrioArray
   *
   * @description
   * does the crio have the key passed, either shallow or deeply nested
   *
   * @param {Array<number|string>|number} key key to test
   * @returns {boolean} does the crio have the key
   */


  CrioArray.prototype.has = function has(key) {
    return _has(key, this);
  };

  /**
   * @function intersection
   * @memberof CrioArray
   *
   * @description
   * find the values in that exist in this and each of the arrays passed
   *
   * @param {...Array<Array>} arrays to find the intersecting values of
   * @returns {CrioArray} array of values that exist in all arrays
   */


  CrioArray.prototype.intersection = function intersection() {
    for (var _len2 = arguments.length, arrays = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
      arrays[_key2] = arguments[_key2];
    }

    if (!arrays.length) {
      return this;
    }

    var indices = [],
        indexOfItem = void 0;

    var reducedArrays = [this].concat(arrays).reduce(function (items, array) {
      if (isArray(array)) {
        array.forEach(function (item) {
          indexOfItem = items.indexOf(item);

          if (~indexOfItem) {
            return indices[indexOfItem]++;
          }

          indices[items.length] = 1;
          items.push(item);
        });
      }

      return items;
    }, []);

    var newLength = arrays.length + 1;

    return new CrioArray(reducedArrays.filter(function (itemIgnored, index) {
      return indices[index] === newLength;
    }));
  };

  /**
   * @function isArray
   * @memberof CrioArray
   *
   * @description
   * is the crio an array
   *
   * @returns {boolean} is the crio an array
   */


  CrioArray.prototype.isArray = function isArray() {
    return true;
  };

  /**
   * @function isObject
   * @memberof CrioArray
   *
   * @description
   * is the crio an object
   *
   * @returns {boolean} is the crio an object
   */


  CrioArray.prototype.isObject = function isObject() {
    return false;
  };

  /**
   * @function join
   * @memberof CrioArray
   *
   * @description
   * join the values in the crio as a string, combined with separator
   *
   * @param {string} [separator] character(s) to place between strings in combination
   * @returns {string} parameters joined by separator in string
   */


  CrioArray.prototype.join = function join(separator) {
    return this.thaw().join(separator);
  };

  /**
   * @function keys
   * @memberof CrioArray
   *
   * @description
   * get the keys of the crio
   *
   * @returns {CrioArray<number>} keys in the crio
   */


  CrioArray.prototype.keys = function keys() {
    return new CrioArray(Object.keys(this).map(Number));
  };

  /**
   * @function last
   * @memberof CrioArray
   *
   * @description
   * take the last n number of items in the array
   *
   * @param {number} [size=1] size of elements to take from end of array
   * @returns {CrioArray}
   */


  CrioArray.prototype.last = function last() {
    var size = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 1;

    return this.slice(this.length - size);
  };

  /**
   * @function map
   * @memberof CrioArray
   *
   * @description
   * map over the array returning the mapped items
   *
   * @param {function} fn the function to map
   * @returns {CrioArray} the mapped array
   */


  CrioArray.prototype.map = function map(fn) {
    var _this2 = this;

    return Array.prototype.map.call(this, function (item, index) {
      return getCrioedObject(fn(item, index, _this2));
    });
  };

  /**
   * @function merge
   * @memberof CrioArray
   *
   * @description
   * merge arrays with the original array
   *
   * @param {Array<number|string>|number|null} key the key to merge into
   * @param {...Array<CrioArray>} objects objects to merge with the crio
   * @returns {CrioArray} merged array
   */


  CrioArray.prototype.merge = function merge(key) {
    for (var _len3 = arguments.length, objects = Array(_len3 > 1 ? _len3 - 1 : 0), _key3 = 1; _key3 < _len3; _key3++) {
      objects[_key3 - 1] = arguments[_key3];
    }

    return objects.reduce(function (mergedObject, object) {
      return _merge(key, getCrioedObject(object), mergedObject);
    }, this);
  };

  /**
   * @function mutate
   * @memberof CrioArray
   *
   * @description
   * work with the array in a mutated way and return the crioed result of that call
   *
   * @param {function} fn function to apply to crio
   * @returns {*} crioed value resulting from the call
   */


  CrioArray.prototype.mutate = function mutate(fn) {
    return getCrioedObject(fn(this.thaw(), this));
  };

  /**
   * @function pluck
   * @memberof CrioArray
   *
   * @description
   * get the values in each object in the collection at key, either shallow or deeply nested
   *
   * @param {string} key key to find value of in collection object
   * @returns {CrioArray} array of plucked values
   */


  CrioArray.prototype.pluck = function pluck(key) {
    var parsedKey = parse(key);

    var arrayToPluck = _get(parsedKey.slice(0, parsedKey.length - 1), this);
    var finalKey = parsedKey.slice(-1);

    return arrayToPluck.map(function (item) {
      return _get(finalKey, item);
    });
  };

  /**
   * @function pop
   * @memberof CrioArray
   *
   * @description
   * get crio based on current crio with last item removed
   *
   * @returns {CrioArray} array with the last value removed
   */


  CrioArray.prototype.pop = function pop() {
    return this.slice(0, this.length - 1);
  };

  /**
   * @function push
   * @memberof CrioArray
   *
   * @description
   * push one to many items to the current crio
   *
   * @param {...Array<*>} items the items to add to the array
   * @returns {CrioArray} array with the values added
   */


  CrioArray.prototype.push = function push() {
    for (var _len4 = arguments.length, items = Array(_len4), _key4 = 0; _key4 < _len4; _key4++) {
      items[_key4] = arguments[_key4];
    }

    return items.length ? this.concat(items) : this;
  };

  /**
   * @function reduce
   * @memberof CrioObject
   *
   * @description
   * reduce the crio down to a single value, starting with initial value
   *
   * @param {function} fn the function to iterate with
   * @param {*} initialValue the initial value of the reduction
   * @returns {*} the reduced value
   */


  CrioArray.prototype.reduce = function reduce(fn, initialValue) {
    var _this3 = this;

    return getCrioedObject(Array.prototype.reduce.call(this, function (value, item, index) {
      return fn(value, item, index, _this3);
    }, initialValue));
  };

  /**
   * @function reduceRight
   * @memberof CrioObject
   *
   * @description
   * reduce the crio down to a single value, starting with initial value, starting from the end of the array
   * and iterating to the start
   *
   * @param {function} fn the function to iterate with
   * @param {*} initialValue the initial value of the reduction
   * @returns {*} the reduced value
   */


  CrioArray.prototype.reduceRight = function reduceRight(fn, initialValue) {
    var _this4 = this;

    return getCrioedObject(Array.prototype.reduceRight.call(this, function (value, item, index) {
      return fn(value, item, index, _this4);
    }, initialValue));
  };

  /**
   * @function reverse
   * @memberof CrioArray
   *
   * @description
   * get the same values, but in reverse order
   *
   * @returns {CrioArray} array with the items reversed in order
   */


  CrioArray.prototype.reverse = function reverse() {
    return new CrioArray([].concat(this).reverse());
  };

  /**
   * @function set
   * @memberof CrioArray
   *
   * @description
   * set the value at the key passed
   *
   * @param {Array<number|string>|number} key key to assign value to
   * @param {*} value value to assign
   * @returns {CrioArray} array with value set at key
   */


  CrioArray.prototype.set = function set(key, value) {
    return _set(key, getCrioedObject(value), this);
  };

  /**
   * @function shift
   * @memberof CrioArray
   *
   * @description
   * get crio based on current crio with first item removed
   *
   * @returns {CrioArray} array with the first item removed
   */


  CrioArray.prototype.shift = function shift() {
    return this.slice(1);
  };

  /**
   * @function sort
   * @memberof CrioArray
   *
   * @description
   * sort the collection by the fn passed
   *
   * @param {function} fn the function to sort based on
   * @returns {CrioArray} array with the items sorted
   */


  CrioArray.prototype.sort = function sort(fn) {
    var clone = [].concat(this);

    clone.sort(fn);

    return new CrioArray(clone);
  };

  /**
   * @function splice
   * @memberof CrioArray
   *
   * @description
   * splice the values into or out of the array
   *
   * @param {number} [start=0] starting index to splice
   * @param {number} [deleteCount=1] length from starting index to removes
   * @param {...Array<*>} items items to insert after delete is complete
   * @returns {CrioArray} array with the value spliced in / out
   */


  CrioArray.prototype.splice = function splice() {
    var clone = [].concat(this);

    clone.splice.apply(clone, arguments);

    return new CrioArray(clone);
  };

  /**
   * @function thaw
   * @memberof CrioArray
   *
   * @description
   * create a plain JS version of the array
   *
   * @returns {Array<*>} plain JS version of the array
   */


  CrioArray.prototype.thaw = function thaw() {
    return _thaw(this);
  };

  /**
   * @function toArray
   * @memberof CrioArray
   *
   * @description
   * convert the array to an array
   *
   * @returns {CrioArray} the array
   */


  CrioArray.prototype.toArray = function toArray() {
    return this;
  };

  /**
   * @function toLocaleString
   * @memberof CrioArray
   *
   * @description
   * convert the array to stringified form
   *
   * @param {function} [serializer] the serialization method to use
   * @param {number} [indent] the number of spaces to indent the values
   * @returns {string} stringified array
   */


  CrioArray.prototype.toLocaleString = function toLocaleString(serializer, indent) {
    return this.toString(serializer, indent);
  };

  /**
   * @function toObject
   * @memberof CrioArray
   *
   * @description
   * convert the crio to an object if it isn't already
   *
   * @returns {CrioObject} new object from the array
   */


  CrioArray.prototype.toObject = function toObject() {
    return this.reduce(function (object, item, index) {
      object[index] = item;

      return object;
    }, new CrioObject({}));
  };

  /**
   * @function toString
   * @memberof CrioArray
   *
   * @description
   * convert the array to stringified form
   *
   * @param {function} [serializer] the serialization method to use
   * @param {number} [indent] the number of spaces to indent the values
   * @returns {string} stringified array
   */


  CrioArray.prototype.toString = function toString(serializer, indent) {
    return stringify(this, serializer, indent);
  };

  /**
   * @function unique
   * @memberof CrioArray
   *
   * @description
   * return the current CrioArray with the duplicate values removed
   *
   * @returns {CrioArray} new crio instance
   */


  CrioArray.prototype.unique = function unique() {
    var hashArray = [],
        newArray = [],
        hasHashCode = false,
        hashCode = void 0,
        storeValue = void 0;

    return this.filter(function (item) {
      hashCode = item ? item.hashCode : undefined;
      hasHashCode = !isUndefined(hashCode);
      storeValue = !~newArray.indexOf(item) && (!hasHashCode || !~hashArray.indexOf(hashCode));

      if (storeValue) {
        newArray.push(item);

        if (hasHashCode) {
          hashArray.push(hashCode);
        }
      }

      return storeValue;
    });
  };

  /**
   * @function unshift
   * @memberof CrioArray
   *
   * @description
   * add items passed to the beginning of the crio array
   *
   * @param {...Array<*>} items items to prepend to the array
   * @returns {CrioArray} array with the items prepended
   */


  CrioArray.prototype.unshift = function unshift() {
    for (var _len5 = arguments.length, items = Array(_len5), _key5 = 0; _key5 < _len5; _key5++) {
      items[_key5] = arguments[_key5];
    }

    return items.length ? new CrioArray([].concat(items, this)) : this;
  };

  /**
   * @function valueOf
   * @memberof CrioArray
   *
   * @description
   * get the array value
   *
   * @returns {CrioArray} the array
   */


  CrioArray.prototype.valueOf = function valueOf() {
    return this;
  };

  /**
   * @function values
   * @memberof CrioArray
   *
   * @description
   * get the values of the array as an array
   *
   * @returns {CrioArray} values in the array
   */


  CrioArray.prototype.values = function values() {
    return getValues(this);
  };

  /**
   * @function xor
   * @memberof CrioArray
   *
   * @description
   * find the values that are the symmetric difference of this and the arrays passed
   *
   * @param {Array<Array>} arrays arrays to find symmetric values in
   * @returns {CrioArray} array of the symmetric differences of all the arrays
   */


  CrioArray.prototype.xor = function xor() {
    for (var _len6 = arguments.length, arrays = Array(_len6), _key6 = 0; _key6 < _len6; _key6++) {
      arrays[_key6] = arguments[_key6];
    }

    if (!arrays.length) {
      return this;
    }

    var indicesToRemove = [],
        indexOfItem = void 0;

    var reducedItems = [this].concat(arrays).reduce(function (items, array) {
      if (isArray(array)) {
        array.forEach(function (item) {
          indexOfItem = items.indexOf(item);

          if (~indexOfItem) {
            indicesToRemove.push(indexOfItem);
          } else {
            items = items.push(item);
          }
        });
      }

      return items;
    }, new CrioArray([]));

    return reducedItems.filter(function (itemIgnored, index) {
      return !~indicesToRemove.indexOf(index);
    });
  };

  _createClass(CrioArray, [{
    key: 'hashCode',
    get: function get() {
      return hashIt(this, true);
    }
  }]);

  return CrioArray;
}(_extendableBuiltin(Array));

export function applyPrototype() {
  Object.keys(ARRAY_FALLBACK_PROTOTYPE_METHODS).forEach(function (key) {
    if (typeof Array.prototype[key] !== 'function') {
      CrioArray.prototype[key] = function () {
        for (var _len7 = arguments.length, args = Array(_len7), _key7 = 0; _key7 < _len7; _key7++) {
          args[_key7] = arguments[_key7];
        }

        return ARRAY_FALLBACK_PROTOTYPE_METHODS[key].apply(ARRAY_FALLBACK_PROTOTYPE_METHODS, [this].concat(args));
      };
    }
  });

  if (typeof Symbol === 'function') {
    if (Symbol.species) {
      Object.defineProperty(CrioArray, Symbol.species, {
        configurable: false,
        enumerable: false,
        get: function get() {
          return CrioArray;
        }
      });
    }

    if (Symbol.iterator) {
      Object.defineProperty(CrioArray.prototype, Symbol.iterator, {
        configurable: false,
        enumerable: false,
        value: createIterator(),
        writable: false
      });
    }

    if (Symbol.unscopables) {
      Object.defineProperty(CrioArray.prototype, Symbol.unscopables, {
        configurable: false,
        enumerable: false,
        value: ARRAY_UNSCOPABLES,
        writable: false
      });
    }
  }
}

export default CrioArray;