'use strict';

exports.__esModule = true;
exports.isUndefined = exports.isString = exports.isObject = exports.isReactElement = exports.isNumber = exports.isFunction = exports.isEqual = exports.isCrio = exports.isArray = undefined;

var _CrioArray = require('./CrioArray');

var _CrioArray2 = _interopRequireDefault(_CrioArray);

var _CrioObject = require('./CrioObject');

var _CrioObject2 = _interopRequireDefault(_CrioObject);

var _constants = require('./constants');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * @function isArray
 *
 * @description
 * is the object passed an array
 *
 * @param {*} object the object to test
 * @returns {boolean} is the object an array
 */
var isArray = exports.isArray = Array.isArray;

/**
 * @function isCrio
 *
 * @description
 * is the object passed a CrioArray or CrioObject
 *
 * @param {*} object the object to test
 * @returns {boolean} is the object a Crio*
 */


// constants
// classes
var isCrio = exports.isCrio = function isCrio(object) {
  return object instanceof _CrioArray2.default || object instanceof _CrioObject2.default;
};

/**
 * @function isEqual
 *
 * @description
 * are the crio objects equal
 *
 * @param {CrioArray|CrioObject} crio crio object to test against
 * @param {*} object object to test equality with crio object for
 * @returns {boolean} are the objects equal
 */
var isEqual = exports.isEqual = function isEqual(crio, object) {
  return isCrio(object) && crio.hashCode === object.hashCode;
};

/**
 * @function isFunction
 *
 * @description
 * is the object passed a function
 *
 * @param {*} object the object to test
 * @returns {boolean} is the object a function
 */
var isFunction = exports.isFunction = function isFunction(object) {
  return typeof object === 'function';
};

/**
 * @function isNumber
 *
 * @description
 * is the object passed a number
 *
 * @param {*} object the object to test
 * @returns {boolean} is the object a number
 */
var isNumber = exports.isNumber = function isNumber(object) {
  return typeof object === 'number';
};

/**
 * @function isReactElement
 *
 * @description
 * is the object passed a react element
 *
 * @param {*} object the object to test
 * @returns {boolean} is the object a react element
 */
var isReactElement = exports.isReactElement = function isReactElement(object) {
  return !!object && object.$$typeof === _constants.REACT_ELEMENT_TYPE;
};

/**
 * @function isObject
 *
 * @description
 * is the object passed a plain object
 *
 * @param {*} object the object to test
 * @returns {boolean} is the object a  plain object
 */
var isObject = exports.isObject = function isObject(object) {
  return !!object && object.constructor === Object && !isReactElement(object);
};

/**
 * @function isString
 *
 * @description
 * is the object passed a string
 *
 * @param {*} object the object to test
 * @returns {boolean} is the object a string
 */
var isString = exports.isString = function isString(object) {
  return typeof object === 'string';
};

/**
 * @function isUndefined
 *
 * @description
 * is the object passed undefined
 *
 * @param {*} object the object to test
 * @returns {boolean} is the object undefined
 */
var isUndefined = exports.isUndefined = function isUndefined(object) {
  return object === void 0;
};